import numpy as np
import sys
from skimage.transform import rescale, warp
from sklearn.metrics import mean_squared_error as mse
from absl import app
from absl import flags


from kernels.hilbert import HilbertGP
from placementcriteria.entropy_CT import Entropy_CT



flags.DEFINE_string('anglefile', "ct_opteven8_chosen.npy",
                    'Path to file containing chosen angles')
FLAGS = flags.FLAGS

def get_integrals(angle, image, center):
    angle = np.deg2rad(angle)
    cos_a, sin_a = np.cos(angle), np.sin(angle)
    R = np.array([[cos_a, sin_a, -center * (cos_a + sin_a - 1)],
                  [-sin_a, cos_a, -center * (cos_a - sin_a - 1)],
                  [0, 0, 1]])
    rotated_ = warp(image, R, clip=False)
    return rotated_.sum(0)

def compute_integrals(angles, image, center):
    y = []
    for angle in angles:
        y.extend(get_integrals(angle, image, center))
    return np.array(y)

def fitGP(chosen, A, K, image, center, S, basisfunctions, w0, filename, Xi, optimize=False):
    Y = compute_integrals(chosen, image, center)
    Eft, Varft, theta = K.approx_2d(np.array(A), Y, S, basisfunctions, w0, optimize=optimize)
    print(theta)
    np.save(filename + "8_eft.npy", Eft)

def main(argv):
    image = np.genfromtxt('data/CT/lungsegment.txt', delimiter=',')
    image = (image - np.min(image)) / np.max(image)
    image = rescale(image, scale=0.25, mode='reflect', multichannel=False)

    # Test Data
    n = image.shape[0]
    [Xi, Xj] = np.meshgrid(np.linspace(0,image.shape[0], n), np.linspace(0, image.shape[0], n))
    S = np.vstack((Xi.ravel(), Xj.ravel())).T

    basisfunctions = 100
    filename = "data/CT/mean_" + FLAGS.anglefile
    center = 63.5
    K = HilbertGP('Matern', v=1)
    w0 = np.log(np.array([8.5, 0.01, 0.1]))
    angles = np.load("data/CT/" + FLAGS.anglefile)
    print(FLAGS.anglefile)
    print(angles)
    sys.stdout.flush()

    # TODO: criterion used only for getting lines, need refactoring
    criterion = Entropy_CT('mean', 8., 0.01, 0.1, m=basisfunctions)
    A = []
    for angle in np.deg2rad(angles):
        A.extend(criterion.get_newlines(angle))
    A = np.array(A)
    fitGP(angles, A, K, image, center, S, basisfunctions, w0, filename, Xi, optimize=True)


if __name__ == "__main__":
    app.run(main)

