import numpy as np

class Stationary():

    def __init__(self, variance=1., lengthscale=1., noise=1.):
        self.lengthscale = lengthscale
        self.variance = variance
        self.noise = noise

    def getw(self, p1, p2):
        return [p2[0] - p1[0], p2[1] - p1[1]]

    def kernel(self, X1, X2):
        kern = np.zeros([len(X1), len(X2)])
        if len(X1) == len(X2):
            for i in range(len(X1)):
                for j in range(i, len(X2)):
                    res = self.k(X1[i], X2[j])
                    kern[i, j] = res
                    kern[j, i] = res
        else:
            for i in range(len(X1)):
                for j in range(len(X2)):
                    kern[i, j] = self.k(X1[i], X2[j])
        return kern.T


    def k(self, X1, X2):
        X1 = np.array(X1)
        X2 = np.array(X2)
        if X1.ndim == 2 and X2.ndim == 2:
            return self.doubleintegralkernel(X1, X2)
        if X1.ndim == 2 and X2.ndim == 1:
            return self.integralkernel(X1, X2)
        if X1.ndim == 1 and X2.ndim == 2:
            return self.integralkernel(X2, X1)
        if X1.ndim == 1 and X2.ndim == 1:
            return self.pointkernel(X1, X2)
        raise Exception('hmm')

    def unscaled_dist(self, X, X2):
        X1sq = np.sum(np.square(X), 1)
        X2sq = np.sum(np.square(X2), 1)
        r2 = -2. * np.dot(X, X2.T) + (X1sq[:, None] + X2sq[None, :])
        r2 = np.clip(r2, 0, np.inf)
        return np.sqrt(r2) / self.lengthscale

    def pointkernel(self, x, xnew):
        x = x[None, :]
        xnew = xnew[None, :]
        return self.kernel_vector(x, xnew)

    def doubleintegralkernel(self, r):
        raise NotImplementedError("implement the covariance function as a fn of r to use this class")

    def integralkernel(self, r):
        raise NotImplementedError("implement the covariance function as a fn of r to use this class")

    def kernel_vector(self, r):
        raise NotImplementedError("implement the covariance function as a fn of r to use this class")

    def fit_GP(self, X, S, Y):
        Kxx = self.kernel(X, X) + self.noise * np.eye(len(X))
        Kxs = self.kernel(X, S)
        Kss = self.kernel(S, S)
        mean = Kxs @ np.linalg.inv(Kxx) @ Y
        Cov = Kss - Kxs @ np.linalg.inv(Kxx) @ Kxs.T
        Cov2 = np.diagonal(Cov)
        return mean, Cov2
