from .entropy import Entropy_BFE, Entropy_numeric
import numpy as np
from shapely.geometry import LineString, Point

class Entropy_CT(Entropy_BFE):
    def __init__(self, summarytype, lengthscale, magnsigma, sigma2, m):
        super().__init__(summarytype, lengthscale, magnsigma, sigma2, m)

        # TODO: remove hardcoding
        self.circle = False
        self.center = np.array([63.5, 63.5])
        self.d = 2
        self.Lt = 4/3 * np.array([128, 128])
        self.start = np.concatenate((np.zeros(128)[:, None],
                                     np.array(range(128))[:, None]), axis=1)
        self.end = np.concatenate(((128 - 1) * np.ones(128)[:, None],
                                   np.array(range(128))[:, None]), axis=1)

    #
    # Get generated lines for certain angle
    #
    # The lines are mapped either to a circle or a square
    # (depending on how sinogram is computed)

    def get_newlines(self, angle):
        # TODO: Remove handcoded values
        def intersection(line, angle):
            p = Point(63.5, 63.5)
            if self.circle == True:
                c = p.buffer((63.5)).boundary
            else:
                c = p.buffer((63.5), cap_style=3).boundary
            l = LineString(line)
            i = c.intersection(l)
            if i.geom_type == 'LineString':
                return line
            if i.geom_type == 'MultiPoint':
                return np.array([[i[0].x, i[0].y], [i[1].x, i[1].y]])
            if i.geom_type == 'Point':
                if line[0][0] >= 0 and line[0][1] >= 0 and line[0][0] < 127 and line[0][1] < 127:
                    return np.array([[line[0][0], line[0][1]], [i.x, i.y]])
                else:
                    return np.array([[i.x, i.y], [line[1][0], line[1][1]]])
            else:
                return line

        def rotate(point, angle):
            px = np.cos(angle) * (point[0] - self.center[0]) - np.sin(angle) * (point[1] - self.center[1]) + self.center[0]
            py = np.sin(angle) * (point[0] - self.center[0]) + np.cos(angle) * (point[1] - self.center[1]) + self.center[1]
            line = np.array([px, py])
            line[np.abs(line) < 1e-12] = 0
            return line

        def getLines(angle):
            l = []
            for i in range(len(self.start)):
                e = rotate(self.end[i], angle)
                s = rotate(self.start[i], angle)
                l_ = intersection(np.array([s, e]), angle)
                if len(l_) > 0:
                    l.append(l_)
            return np.array(l)


        l = getLines(angle)
        return l

    def initialize(self, angles):
        return self.get_newlines(np.deg2rad(angles[0])), angles[0], np.delete(angles, 0)


    # Find the next optimal sensor (angle) and the lines generated by that
    def get_nextlines(self, A, angles):
        A = np.array(A)
        L_inv = self.K_inverse(A)

        scores = []
        lines = []
        print("angles")
        print(angles)
        for angle in np.deg2rad(angles):
            print("curangle")
            print(angle)
            newlines = self.get_newlines(angle)
            sig = self.sigma(newlines, L_inv)
            print("sig")
            print(sig)
            s = self.score(self.entropy(sig))
            scores.append(s)
            lines.append(newlines)

        scores = np.round(np.array(scores), decimals=5)
        print(scores)
        maxind = np.argmax(scores)
        chosenangle = angles[maxind]
        return lines[maxind], chosenangle, np.delete(angles, maxind)

class Entropy_CT2(Entropy_numeric):
    def __init__(self, summarytype, kernel):
        super().__init__(summarytype, kernel)

        # TODO: remove hardcoding
        self.circle = False
        self.center = np.array([63.5, 63.5])
        self.d = 2
        self.start = np.concatenate((np.zeros(128)[:, None],
                                     np.array(range(128))[:, None]), axis=1)
        self.end = np.concatenate(((128 - 1) * np.ones(128)[:, None],
                                   np.array(range(128))[:, None]), axis=1)

    #
    # Get generated lines for certain angle
    #
    # The lines are mapped either to a circle or a square
    # (depending on how sinogram is computed)

    def get_newlines(self, angle):
        # TODO: Remove handcoded values
        def intersection(line, angle):
            p = Point(63.5, 63.5)
            if self.circle == True:
                c = p.buffer((63.5)).boundary
            else:
                c = p.buffer((63.5), cap_style=3).boundary
            l = LineString(line)
            i = c.intersection(l)
            if i.geom_type == 'LineString':
                return line
            if i.geom_type == 'MultiPoint':
                return np.array([[i[0].x, i[0].y], [i[1].x, i[1].y]])
            if i.geom_type == 'Point':
                if line[0][0] >= 0 and line[0][1] >= 0 and line[0][0] < 127 and line[0][1] < 127:
                    return np.array([[line[0][0], line[0][1]], [i.x, i.y]])
                else:
                    return np.array([[i.x, i.y], [line[1][0], line[1][1]]])
            else:
                return line

        def rotate(point, angle):
            px = np.cos(angle) * (point[0] - self.center[0]) - np.sin(angle) * (point[1] - self.center[1]) + \
                 self.center[0]
            py = np.sin(angle) * (point[0] - self.center[0]) + np.cos(angle) * (point[1] - self.center[1]) + \
                 self.center[1]
            line = np.array([px, py])
            line[np.abs(line) < 1e-12] = 0
            return line

        def getLines(angle):
            l = []
            for i in range(len(self.start)):
                e = rotate(self.end[i], angle)
                s = rotate(self.start[i], angle)
                l_ = intersection(np.array([s, e]), angle)
                if len(l_) > 0:
                    l.append(l_)
            return np.array(l)

        l = getLines(angle)
        return l

    def initialize2(self, angles):
        return self.get_newlines(np.deg2rad(angles[0])), angles[0], np.delete(angles, 0)

    def initialize(self, angles):
        scores = []
        print("init")
        print(angles)
        for angle in np.deg2rad(angles):
            newlines = self.get_newlines(angle)
            s = self.K.kernel(newlines, newlines)
            scores.append(self.score(s))

        ind = np.argmax(scores)
        lines = self.get_newlines(np.deg2rad(angles[ind]))
        angle = angles[ind]
        angles = np.delete(angles, ind)
        return lines, angle, angles

    # Find the next optimal sensor (angle) and the lines generated by that
    def get_nextlines(self, A, angles):
        A = np.array(A)
        L_inv = self.K_inverse(A)

        scores = []
        lines = []
        for angle in np.deg2rad(angles):
            newlines = self.get_newlines(angle)
            sig = self.sigma(newlines, A, L_inv)
            s = self.score(self.entropy(sig))
            scores.append(s)
            lines.append(newlines)

        scores = np.round(np.array(scores), decimals=5)
        maxind = np.argmax(scores)
        chosenangle = angles[maxind]
        return lines[maxind], chosenangle, np.delete(angles, maxind)