from .entropy import Entropy_numeric
import numpy as np

class Entropy_ultrasound(Entropy_numeric):
    def __init__(self, summarytype, kernel, second=False, third=False):
        super().__init__(summarytype, kernel)
        print(summarytype)
        self.second = second
        self.third = third

    def get_newlines(self, points1, points2):
        lines = []
        for point1 in points1:
            for point2 in points2:
                if not np.array_equal(point1, point2):
                    lines.append([point1, point2])
                    if self.second == True:
                        lines.append([point1, point2 + [self.K.period, 0]])
                        lines.append([point1, point2 + [-self.K.period, 0]])
                    if self.third == True:
                        lines.append([point1, point2 + [2 * self.K.period, 0]])
                        lines.append([point1, point2 + [-2 * self.K.period, 0]])
        return np.array(lines)

    def initial_point(self, points1, points2):
        scores = []
        indices = []
        for i in range(len(points1)):
            for j in range(len(points2)):
                if not np.array_equal(points1[i], points2[j]):
                    l = self.get_newlines(points1[i][None, :], points2[j][None, :])
                    s = self.score(self.entropy(self.K.kernel(l, l)))
                    scores.append(s)
                    indices.append([i, j])
        scores = np.round(scores, decimals=5)
        return indices[np.argmax(np.array(scores))]

    def initialize(self, S, E):
        initial = self.initial_point(S, E)
        starts = [S[initial[0]]]
        ends = [E[initial[1]]]
        A = self.get_newlines(S[initial[0]][None, :], E[initial[1]][None, :])
        S = np.delete(S, initial[0], axis=0)
        E = np.delete(E, initial[1], axis=0)
        return list(A), S, E, starts, ends

    def get_nextlines(self, A, S, E, starts, ends):
        Kaa_inv = self.K_inverse(np.array(A))

        # Scores for adding a new start point
        scores_s, lines_s = self.scores(Kaa_inv, A, S, ends)
        # Scores for adding a new end point
        scores_e, lines_e = self.scores2(Kaa_inv, A, E, starts)

        # Find best point
        if not scores_s.size:
            ends.append(E[np.argmax(scores_e)])
            E = np.delete(E, np.argmax(scores_e), axis=0)
            return lines_e[np.argmax(scores_e)], S, E, starts, ends
        elif not scores_e.size:
            starts.append(S[np.argmax(scores_s)])
            S = np.delete(S, np.argmax(scores_s), axis=0)
            return lines_s[np.argmax(scores_s)], S, E, starts, ends
        elif np.isclose(np.max(scores_e), np.max(scores_s), rtol=1e-04, atol=1e-05, equal_nan=False):
            ends.append(E[np.argmax(scores_e)])
            E = np.delete(E, np.argmax(scores_e), axis=0)
            return lines_e[np.argmax(scores_e)], S, E, starts, ends
        elif np.max(scores_e) > np.max(scores_s):
            ends.append(E[np.argmax(scores_e)])
            E = np.delete(E, np.argmax(scores_e), axis=0)
            return lines_e[np.argmax(scores_e)], S, E, starts, ends
        else:
            starts.append(S[np.argmax(scores_s)])
            S = np.delete(S, np.argmax(scores_s), axis=0)
            return lines_s[np.argmax(scores_s)], S, E, starts, ends
