import numpy as np
import math

class MutualInformation():
    def __init__(self, summarytype):
        self.summarytype = summarytype
        self.decimals = 5

    def mutual_inf(self, sigmaA, sigmaV):
        return sigmaA / sigmaV

    def score(self, mutual_inf):
        if self.summarytype == 'mean':
            s = np.mean(mutual_inf)
        if self.summarytype == 'min':
            s = np.min(mutual_inf)
        if self.summarytype == 'max':
            s = np.max(mutual_inf)
        return s

    def scores(self, Kaa_inv, chosenlines, sensors, chosensensors, all_lines):
        scores = []
        lines = []
        for x in sensors:
            newlines = self.get_newlines([x], chosensensors)
            sigA = self.sigma(newlines, chosenlines, Kaa_inv)
            sigV = self.sigma(all_lines, newlines)
            s = self.score(self.mutual_inf(sigA, sigV))
            scores.append(s)
            lines.append(newlines)
        scores = np.round(np.array(scores), decimals=self.decimals)
        return scores, np.array(lines)

    def sigma(self, lines, chosensenlines):
        raise NotImplementedError("implement to use this class")

    def get_newlines(self, sensor, chosensensors):
        raise NotImplementedError("implement to use this class")


class MI_numeric(MutualInformation):
    def __init__(self, summarytype, kernel):
        super().__init__(summarytype)
        self.K = kernel

    def sigma(self, X, S, Kxx_inv=None):
        sigmas = []
        if Kxx_inv is None:
            Kxx_inv = self.K_inverse(X)

        for S_ in S:
            Kxs = self.K.kernel(X, [S_])
            Kss = self.K.kernel([S_], [S_])
            Cov = Kss - Kxs @ Kxx_inv @ Kxs.T
            sigmas.append(np.squeeze(Cov))
        return np.array(sigmas)

    def K_inverse(self, A):
        Kaa = self.K.kernel(A, A) + self.K.noise * np.eye(len(A))
        return np.linalg.inv(Kaa)