from .mi import MI_numeric
import numpy as np

class MI_toy(MI_numeric):
    def __init__(self, summarytype, kernel, sigma2=1.):
        super().__init__(summarytype, kernel, sigma2)

    def get_all_lines(self, points):
        all_lines = []
        for i in range(len(points)):
            for j in range(i, len(points)):
                point1 = points[i]
                point2 = points[j]
                if not (np.array_equal(point1, point2)):
                    all_lines.append([point1, point2])
        return np.array(all_lines)

    def get_newlines(self, point, sensors):
        lines = []
        point = np.squeeze(point)
        for point2 in sensors:
            lines.append([point, point2])
        return lines

    def delete_from_alllines(self, all_lines, lines):
        notused = np.copy(all_lines)
        for line in lines:
            locs = np.logical_and(notused[:, 0] == [line[0]], notused[:, 1] == [line[1]])
            i = np.argmax(np.logical_and(locs[:, 0] == 1, locs[:, 1] == 1))
            notused = np.delete(notused, i, axis=0)

        return notused

    def initial_line(self, all_lines):
        scores = []
        for line in all_lines:
            line = line[None, :]
            s = self.sigma(line, line)
            s2 = self.sigma(self.delete_from_alllines(all_lines, line), line)
            scores.append(s / s2)
        return np.argmax(np.array(scores))

    def delete_point(self, points, point):
        i = np.argmax(np.logical_and(points[:, 0] == [point[0]], points[:, 1] == [point[1]]))
        return np.delete(points, i, axis=0)

    def initialize(self, points):
        all_lines = self.get_all_lines(points)
        initial = self.initial_line(all_lines)
        line = all_lines[initial]

        sensors = [line[0], line[1]]
        points = self.delete_point(points, line[0])
        points = self.delete_point(points, line[1])
        return [line], sensors, points, np.delete(all_lines, initial, axis=0)


    def get_nextlines(self, A, chosen, points, all_lines, scores_updated, scores):
        Kaa_inv = self.K_inverse(np.array(A))
        while not scores_updated[np.argmax(scores)]:
            i = np.argmax(scores)
            point = points[i]
            newlines = self.get_newlines(point, chosen)
            sigA = self.sigma(A, newlines, Kaa_inv)
            sigV = self.sigma(self.delete_from_alllines(all_lines, newlines), newlines)
            scores[i] = self.score(self.mutual_inf(sigA, sigV))
            scores_updated[i] = True

        ind = np.argmax(scores)
        newlines = self.get_newlines(points[ind], chosen)
        all_lines = self.delete_from_alllines(all_lines, newlines)
        chosen = points[ind]
        scores = np.delete(scores, ind)
        scores_updated = np.zeros(len(scores))
        points = np.delete(points, ind, axis=0)

        return newlines, chosen, points, all_lines, scores, scores_updated


