import numpy as np
from sklearn.metrics import mean_squared_error as mse
import sys
from kernels.numeric import Numeric
from kernels.analytic_RQ import RQ
from placementcriteria.entropy_toy import Entropy_toy
from placementcriteria.mi_toy import MI_toy


import tensorflow as tf

flags = tf.app.flags
flags.DEFINE_string('criter', "entropy",
                    'entropy/mi')
flags.DEFINE_string('summarytype', "min",
                    'min/mean/max')
flags.DEFINE_integer('grid', "5",
                    'gridsize')
FLAGS = flags.FLAGS

def error(real, predicted):
    return np.sqrt(mse(real, predicted))

def get_integral_values_for_function(lines, f):
    def F_orig(S):
        x = S[:, 0]
        y = S[:, 1]
        return f(x, y)

    def r(t, p1, p2):
        return np.array([(1 - t) * p1[0] + t * p2[0],
                         (1 - t) * p1[1] + t * p2[1]])

    def delta(point1, point2):
        return np.sqrt((point2[0] - point1[0]) ** 2 \
                       + (point2[1] - point1[1]) ** 2)

    n = 50
    integrals = []
    for line in lines:
        point1 = line[0]
        point2 = line[1]
        t = np.linspace(0., 1., n)
        res = 0
        for i in range(1, len(t)):
            res += F_orig(r(np.reshape(t[i], (1,)), point1, point2).T) \
                   * np.sqrt((r(t[i], point1, point2)[0] - r(t[i - 1], point1, point2)[0]) ** 2 \
                             + (r(t[i], point1, point2)[1] - r(t[i - 1], point1, point2)[1]) ** 2)
        integrals.append(np.sum(res))

    return np.array(integrals)


def fitGP_multiple(K, X, S, functions, Xi, Xj):
    Kxx = K.kernel(X, X) + K.noise * np.eye(len(X))
    Kxs = K.kernel(X, S)

    errors = []
    for i in range(len(functions)):
        Y = get_integral_values_for_function(X, functions[i])
        Y = Y + np.random.normal(0, np.sqrt(K.noise), len(Y))
        Y = Y[:, None]
        test_y = functions[i](Xi, Xj).reshape(Xi.shape[0] * Xi.shape[0])
        mean = Kxs @ np.linalg.inv(Kxx) @ Y
        rms = error(test_y, mean)
        errors.append(rms)
    return np.array(errors)


def entropy(t, allpoints,  functions, test_x, Xi, Xj, filename):
    K = RQ(lengthscale=1., variance=1., noise=0.001)

    print(t)
    criterion = Entropy_toy(t, K)
    A, chosen, points = criterion.initialize(allpoints)
    errors = []

    for k_ in range(h - 1):
        print(k_)
        newlines, chosen_, points = criterion.get_nextlines(A, chosen, points)
        A.extend(newlines)
        chosen.append(chosen_)

        # Fit GP
        rms = fitGP_multiple(K, np.array(A), test_x, functions, Xi, Xj)
        errors.append(rms)

    res = np.array(errors).T
    np.save(filename + t + "_error.npy", res)

def mi(t, allpoints,  functions, test_x, Xi, Xj, filename):
    K = RQ(lengthscale=1., variance=1., noise=0.1)
    print(t)
    criterion = MI_toy(t, K)
    A, chosen, points, all_lines = criterion.initialize(allpoints)
    errors = []
    scores = np.ones(len(points)) * 100000
    scores_updated = np.zeros(len(points))

    for k_ in range(h - 1):
        print(k_)
        sys.stdout.flush()
        newlines, chosen_, points, all_lines, scores, scores_updated = criterion.get_nextlines(A, chosen, points,
                                                                                               all_lines,
                                                                                               scores_updated,
                                                                                               scores)

        A.extend(newlines)
        chosen.append(chosen_)

        # Fit GP
        rms = fitGP_multiple(K, np.array(A), test_x, functions, Xi, Xj)
        errors.append(rms)

    res = np.array(errors)
    np.save(filename + t + "_error.npy", res)
    np.save(filename + t + "_chosen.npy", chosen)



if __name__ == "__main__":

    # Test Data
    obs = 15
    gridstart = 0
    gridend = 5

    # f = lambda x,y: (-(y+47) * np.sin(np.sqrt(np.abs((x/2)+(y+47))))-x*np.sin(np.sqrt(np.abs(x-(y+47))))) / 100
    f1 = lambda x, y: -np.abs(np.sin(x) * np.cos(y) * np.exp(np.abs(1 - np.sqrt(x ** 2 + y ** 2) / np.pi))) / 2
    f2 = lambda x, y: ((x ** 2 + y - 11) ** 2 + (x + y ** 2 - 7) ** 2) / 600
    f3 = lambda x, y: (np.sin(3 * np.pi * x) ** 2 + (x - 1) ** 2 * (1 + np.sin(2 * np.pi * y) ** 2) + (y - 1) ** 2 * (1 + np.sin(2 * np.pi * y) ** 2)) / 50
    f4 = lambda x, y: (0.26 * (x ** 2 + y ** 2) - 0.48 * x * y) / 5
    f5 = lambda x, y: ((x + 2 * y - 7) ** 2 + (2 * x + y - 5) ** 2) / 120
    f6 = lambda xi, xj: np.sin(xi / 2) ** 2 * np.sin(xj / 2) ** 2
    f7 = lambda xi, xj: np.cos(xi / 2) ** 2 * np.sin(xj / 2)
    f8 = lambda x, y: ((x ** 2 - y - 11) ** 2 + (x - y ** 2 - 7) ** 2) / 1000
    f9 = lambda xi, xj: np.sin(xi) * np.cos(xj)
    f10 = lambda x, y: (-20 * np.exp(-0.2 * np.sqrt(0.5 * (x ** 2 + y ** 2))) - np.exp(0.5 * (np.cos(2 * np.pi * x) + np.cos(2 * np.pi * y))) + np.exp(1) + 20) / 14
    functions = [f1, f2, f3, f4, f5, f6, f7, f8, f9, f10]

    [Xi, Xj] = np.meshgrid(np.linspace(gridstart, gridend, obs), np.linspace(gridstart, gridend, obs))
    S = np.vstack((Xi.ravel(), Xj.ravel())).T
    test_x = S

    # Possible locations for sensors
    arrangement = 'all'  # border or all
    p = FLAGS.grid
    h = 12

    V = np.array(np.meshgrid(np.linspace(gridstart, gridend, p), np.linspace(gridstart, gridend, p)))
    V = V.reshape(2, p * p).T
    points = V

    if arrangement == 'border':
        points = []
        for point in V:
            if point[1] < 0.1:
                points.append(point)
            if point[0] < 0.1 and 0 < point[1] < 4.9:
                points.append(point)
            if point[1] > 4.9:
                points.append(point)
            if point[0] > 4.9 and 0.1 < point[1] < 5:
                points.append(point)

    points = np.array(points)
    criter = FLAGS.criter
    type = FLAGS.summarytype

    if criter == 'entropy':
        filename = "data/toy/toy_entropy_" + arrangement + str(p)
        entropy(type, points,  functions, test_x, Xi, Xj, filename)

    if criter == 'mi':
        filename = "data/toy/toy_mi_" + arrangement + str(p)
        mi(type, points,  functions, test_x, Xi, Xj, filename)



